package MBF::Test2;

#<<<
use     strict;
use     5.007;
use     vars        qw( $VERSION );
#>>>

# The equivalent of "use warnings" pre-5.006.
local $WARNING = 1;

$VERSION = '0.000_001';

=pod

=head1 NAME

MBF::Test2 - Second test module for Module::Build::Functions

=head1 AUTHORS

Curtis Jewell E<lt>perl@csjewell.fastmail.usE<gt>

Curtis Jewell E<lt>csjewell@cpan.orgE<gt>

Curtis Jewell C<< <csjewell@cpan.org> >>

=head1 LICENSE

proprietary

=head1 END_TEST

=cut

1;